% this is the script used for the simulations shown in Fig. 8 of the paper 
% "Cingulate dependent social risk assessment in rats, Yingying Han1*, Rune Bruls1*, Efe Soyman1, Rajat Mani Thomas1, Vasiliki Pentaraki1,a, Naomi Jelinek1,b, Mirjam Heinemans1,2, Iege Bassez1,c,  Sam Verschooren1,c, Illanah Pruis1,a, Thijs Van Lierde1,c, Nathaly Carrillo1, Valeria Gazzola1,2, Maria Carrillo1+ and Christian Keysers1,2+
% Each section draws one of the pannels
% script (C) Christian Keysers, version 16 May 2019
% Note: the script uses the colorscale 'fireice.m' that must be in the
% directory to have the same color scales as in the paper

%% generates 1000 time points, including 100 s on 100s off danger clues.
rng('default');
rng(1);

n=1000; %number of time points

danger=repmat([ones(1,100) zeros(1,100)],1,5);
t=[1:n];
noise1=normrnd(0,1,1,1000); % noise on whether or not it is really danger
noise2=normrnd(0,1,1,1000);

internal1=danger+noise1;
internal2=danger+noise2;


%% Generates Fig 8A a difference for a very specific threshold
figure(1)
b=1;
c=0.7; %specific threshold set
hits=NaN(size(c));
fas=NaN(size(c));
for i=1:length(c)
    freeze1(1)=internal1(1)>c(i);
    freeze2(1)=internal2(1)>c(i);
    for j=2:n
        freeze1(j)=internal1(j)+b*(freeze2(j-1)-0.5)>c(i);
        freeze2(j)=internal2(j)+b*(freeze1(j-1)-0.5)>c(i);
    end
    hits(i)=sum(freeze1(danger==1))/sum(danger==1)
    fas(i)=sum(freeze1(danger==0))/sum(danger==0)
end

plot(t,danger,t,internal1+4,t,repmat(c,1,1000)+4)

%% Generates Fig 8B a difference for a very specific threshold
figure(2)
subplot(2,1,1) % freezing based on personal danger signal alone
imagesc([internal1>0.7])
colormap([1 1 1;1 0 0])
subplot(2,1,2) % freezing based on the socially informed freezing calculated above 
imagesc([freeze1])
colormap([1 1 1;1 0 0])

%% Generates Figure 8C: ROC curves for various b values
r=1
b=[0 0.5 1 1.5];% bias of how much you get influenced by the freezing of the other
nl=1;% noise level

internal1=danger+r*nl*noise1;
internal2=danger+nl*noise2;

c=[-5:.1:5].*nl*r;
c=fliplr(c);

% first calculation of the decisions based on internal signal alone
hit=NaN(size(c)); 
fa=NaN(size(c));



for i=1:length(c)
    freeze1=internal1>c(i);
    freeze2=internal2>c(i);
    hit(i)=sum(freeze1(danger==1))/sum(danger==1);
    fa(i)=sum(freeze1(danger==0))/sum(danger==0);
end
[0 trapz(fa,hit)]
figure(3)
plot(fa,hit,'r')
hold on

% second calculation of the decision including the past freezing of the
% other rat (apendix s for social)
for bi=2:4
hits=NaN(size(c));
fas=NaN(size(c));
for i=1:length(c)
    freeze1(1)=internal1(1)>c(i);
    freeze2(1)=internal2(1)>c(i);
    for j=2:n
        freeze1(j)=internal1(j)+b(bi)*(freeze2(j-1)-0.5)>c(i);
        freeze2(j)=internal2(j)+b(bi)/r*(freeze1(j-1)-0.5)>c(i);
    end
    hits(i)=sum(freeze1(danger==1))/sum(danger==1);
    fas(i)=sum(freeze1(danger==0))/sum(danger==0);
end

plot(fas,hits,'g')
[b(bi) trapz(fas,hits)]
end
hold off

% finally, provide the AUC for [social individual]
benefit=(trapz(fas,hits) - trapz(fa,hit))/(trapz(fa,hit)-0.5)




%% plots Fig. 8D: the advantage of mutual influences in cases of symetric access to danger signals
% looping through bias and noise level (takes a good 10minutes, reduces av for faster runing)
av=100; % number of random data simulations per b and nl level. Used 100 for paper, but use 20 for faster running but less stable
b=[0:.1:2];% bias of how much you get influenced by the freezing of the other
nl=[0:.2:10];% noise level

benefitb=NaN(av,length(b),length(nl));

for z=1:av
noise1=normrnd(0,1,1,1000);
noise2=normrnd(0,1,1,1000);
   
for ib=1:length(b) 
    for inl=1:length(nl) 

internal1=danger+nl(inl)*noise1;
internal2=danger+nl(inl)*noise2;

c=[-5:.1:5].*nl(inl);
c=fliplr(c);

% first calculation of the decisions based on internal signal alone
hit=NaN(size(c)); 
fa=NaN(size(c));

for i=1:length(c)
    freeze1=internal1>c(i);
    freeze2=internal2>c(i);
    hit(i)=sum(freeze1(danger==1))/sum(danger==1);
    fa(i)=sum(freeze1(danger==0))/sum(danger==0);
end
% figure(4)
% plot(fa,hit,'r')
% hold on

% second calculation of the decision including the past freezing of the
% other rat (apendix s for social)
hits=NaN(size(c));
fas=NaN(size(c));
for i=1:length(c)
    freeze1(1)=internal1(1)>c(i);
    freeze2(1)=internal2(1)>c(i);
    for j=2:n
        freeze1(j)=internal1(j)+b(ib)*(freeze2(j-1)-0.5)>c(i);
        freeze2(j)=internal2(j)+b(ib)*(freeze1(j-1)-0.5)>c(i);
    end
    hits(i)=sum(freeze1(danger==1))/sum(danger==1);
    fas(i)=sum(freeze1(danger==0))/sum(danger==0);
end

% plot(fas,hits,'g')
% hold off

% finally, provide the AUC for [social individual]
benefitb(z,ib,inl)=(trapz(fas,hits) - trapz(fa,hit))/(trapz(fa,hit)-0.5);
    end
end

end

figure(4)
imagesc(b,nl,squeeze(median(benefitb,1))',[-0.5 .5])
colormap(fireice)

%% Figure 8E and 8F: plots the advantage of mutual influences in cases where the animal has less access than the other
% note that we modulate the connection with noise level
r=2 % this means one animal has 2x the noise of the other
av=100; %number of random data generated for each pixel of the plot. This influences running time 
b=[0:.1:2];% bias of how much you get influenced by the freezing of the other
nl=[0:.2:10];% noise level

benefitr1=NaN(av,length(b),length(nl));
benefitr2=NaN(av,length(b),length(nl));

for z=1:av
    noise1=normrnd(0,1,1,1000);
    noise2=normrnd(0,1,1,1000);

for ib=1:length(b) 
    for inl=1:length(nl) 

internal1=danger+r*nl(inl)*noise1;
internal2=danger+nl(inl)*noise2;

c=[-5:.1:5].*nl(inl)*r;
c=fliplr(c);

% first calculation of the decisions based on internal signal alone
hit=NaN(size(c)); 
fa=NaN(size(c));



for i=1:length(c)
    freeze1=internal1>c(i);
    freeze2=internal2>c(i);
    hit1(i)=sum(freeze1(danger==1))/sum(danger==1);
    fa1(i)=sum(freeze1(danger==0))/sum(danger==0);
    hit2(i)=sum(freeze2(danger==1))/sum(danger==1);
    fa2(i)=sum(freeze2(danger==0))/sum(danger==0);
end
% figure(4)
% plot(fa,hit,'r')
% hold on

% second calculation of the decision including the past freezing of the
% other rat (apendix s for social)
hits1=NaN(size(c));
fas1=NaN(size(c));
hits2=NaN(size(c));
fas2=NaN(size(c));
for i=1:length(c)
    freeze1(1)=internal1(1)>c(i);
    freeze2(1)=internal2(1)>c(i);
    for j=2:n
        freeze1(j)=internal1(j)+b(ib)*(freeze2(j-1)-0.5)>c(i);
        freeze2(j)=internal2(j)+b(ib)/r*(freeze1(j-1)-0.5)>c(i);
    end
    hits1(i)=sum(freeze1(danger==1))/sum(danger==1);
    fas1(i)=sum(freeze1(danger==0))/sum(danger==0);
    hits2(i)=sum(freeze2(danger==1))/sum(danger==1);
    fas2(i)=sum(freeze2(danger==0))/sum(danger==0);
end

% plot(fas,hits,'g')
% hold off

% finally, provide the AUC for [social individual]
benefitr1(z,ib,inl)=(trapz(fas1,hits1) - trapz(fa1,hit1))/(trapz(fa1,hit1)-0.5);
benefitr2(z,ib,inl)=(trapz(fas2,hits2) - trapz(fa2,hit2))/(trapz(fa2,hit2)-0.5);
    end
end
end

figure(5)
imagesc(b,nl,squeeze(median(benefitr1,1))',[-0.5 0.5])%%
colormap(fireice)
figure(6)
imagesc(b,nl,squeeze(median(benefitr2,1))',[-0.5 0.5])%%
colormap(fireice)


%% Figure 8G and 8H:plots the advantage of mutual influences in cases where the animal has less access than the other
% note that we modulate the connection with noise level
r=3 % this means one animal has 3x the noise of the other
av=20;
b=[0:.1:2];% bias of how much you get influenced by the freezing of the other
nl=[0:.2:10];% noise level

benefitr31=NaN(av,length(b),length(nl));
benefitr32=NaN(av,length(b),length(nl));

for z=1:av
    noise1=normrnd(0,1,1,1000);
    noise2=normrnd(0,1,1,1000);

for ib=1:length(b) 
    for inl=1:length(nl) 

internal1=danger+r*nl(inl)*noise1;
internal2=danger+nl(inl)*noise2;

c=[-5:.1:5].*nl(inl)*r;
c=fliplr(c);

% first calculation of the decisions based on internal signal alone
hit=NaN(size(c)); 
fa=NaN(size(c));



for i=1:length(c)
    freeze1=internal1>c(i);
    freeze2=internal2>c(i);
    hit1(i)=sum(freeze1(danger==1))/sum(danger==1);
    fa1(i)=sum(freeze1(danger==0))/sum(danger==0);
    hit2(i)=sum(freeze2(danger==1))/sum(danger==1);
    fa2(i)=sum(freeze2(danger==0))/sum(danger==0);
end
% figure(4)
% plot(fa,hit,'r')
% hold on

% second calculation of the decision including the past freezing of the
% other rat (apendix s for social)
hits1=NaN(size(c));
fas1=NaN(size(c));
hits2=NaN(size(c));
fas2=NaN(size(c));
for i=1:length(c)
    freeze1(1)=internal1(1)>c(i);
    freeze2(1)=internal2(1)>c(i);
    for j=2:n
        freeze1(j)=internal1(j)+b(ib)*(freeze2(j-1)-0.5)>c(i);
        freeze2(j)=internal2(j)+b(ib)/r*(freeze1(j-1)-0.5)>c(i);
    end
    hits1(i)=sum(freeze1(danger==1))/sum(danger==1);
    fas1(i)=sum(freeze1(danger==0))/sum(danger==0);
    hits2(i)=sum(freeze2(danger==1))/sum(danger==1);
    fas2(i)=sum(freeze2(danger==0))/sum(danger==0);
end

% plot(fas,hits,'g')
% hold off

% finally, provide the AUC for [social individual]
benefitr31(z,ib,inl)=(trapz(fas1,hits1) - trapz(fa1,hit1))/(trapz(fa1,hit1)-0.5);
benefitr32(z,ib,inl)=(trapz(fas2,hits2) - trapz(fa2,hit2))/(trapz(fa2,hit2)-0.5);
    end
end
end

figure(7)
imagesc(b,nl,squeeze(median(benefitr31,1))',[-0.5 0.5])%%
colormap(fireice)
figure(8)
imagesc(b,nl,squeeze(median(benefitr32,1))',[-0.5 0.5])%%
colormap(fireice)



